'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var charset_to_unicodeRanges = (charset => {
  if (charset.length === 0) return [];
  let charCodeStart = charset[0].charCodeAt(0);
  let charCodePrev;
  let charCodeNext;
  let isInRange = false;
  const results = [];

  for (let i = 1; i < charset.length; i += 1) {
    charCodePrev = charset[i - 1].charCodeAt(0);
    charCodeNext = charset[i].charCodeAt(0);

    if (charCodeNext === charCodePrev + 1) {
      isInRange = true;
    } else if (isInRange) {
      results.push({
        from: charCodeStart,
        to: charCodePrev
      });
      charCodeStart = charCodeNext;
      isInRange = false;
    } else {
      results.push({
        from: charCodeStart,
        to: charCodeStart
      });
      charCodeStart = charCodeNext;
    }
  }

  if (isInRange) {
    results.push({
      from: charCodeStart,
      to: charCodeNext
    });
  } else {
    results.push({
      from: charCodeStart,
      to: charCodeStart
    });
  }

  return results;
});

var compact_unicodeRanges = ((exactRanges, unusedCharset) => {
  if (exactRanges.length === 0) return exactRanges;
  let rangeCurr = exactRanges[0];
  const results = [rangeCurr];

  for (let i = 1; i < exactRanges.length; i += 1) {
    const rangeNext = exactRanges[i];
    let shouldMerge = true;

    for (let charCode = rangeCurr.to + 1; charCode < rangeNext.from; charCode += 1) {
      const char = String.fromCharCode(charCode);

      if (!unusedCharset.includes(char)) {
        results.push(rangeNext);
        rangeCurr = rangeNext;
        shouldMerge = false;
        break;
      }
    }

    if (shouldMerge) {
      rangeCurr.to = rangeNext.from;
    }
  }

  return results;
});

var str_to_charset = (str => str.split('').sort());

var unicodeRanges_to_cssStr = (cfg => {
  const delim = (cfg === null || cfg === void 0 ? void 0 : cfg.space) ? ', ' : ',';
  const u = (cfg === null || cfg === void 0 ? void 0 : cfg.upper) ? 'U' : 'u';

  const getUnicodeStr = range => {
    const left = range.from.toString(16);

    if (range.from === range.to) {
      return `${u}+${range.from.toString(16)}`;
    }

    const right = range.to.toString(16);
    let leftPad = left.padStart(right.length, '0');
    let wildcards = '';

    for (let i = right.length - 1; i >= 0; i -= 1) {
      const rightChar = right[i];
      const leftChar = leftPad[i];

      if (leftChar === '0' && rightChar === 'f') {
        leftPad = leftPad.slice(0, -1);
        wildcards += '?';
      } else if (leftChar === rightChar) {
        break;
      } else {
        return `${u}+${left}-${right}`;
      }
    }

    return `${u}+${leftPad}${wildcards}`;
  };

  return ranges => {
    if (!ranges.length) {
      return undefined;
    }

    if (ranges[0].from === 0 && ranges[0].to > 1114111) {
      return null;
    }

    let result = getUnicodeStr(ranges[0]);

    for (let i = 1; i < ranges.length; i += 1) {
      result += delim;
      result += getUnicodeStr(ranges[i]);
    }

    return result;
  };
});

exports.charsetToUnicodeRanges = charset_to_unicodeRanges;
exports.compactUnicodeRanges = compact_unicodeRanges;
exports.strToCharset = str_to_charset;
exports.unicodeRangesToCssStr = unicodeRanges_to_cssStr;
